#include <stdio.h>

/*
 * zunaechst einmal eine triviale Parameteruebergabe;
 * die 2 muesste beim Parameter-Typ nicht stehen, die 3 hingegen schon,
 * da sie zur Adressierung notwendig ist;
 * Groesse 2 (der 1. Dim.) muss bei tats. uebergebenem Array
 * nicht stimmen! Ausserdem koennen auch Arrays mit anderer zweiter
 * Dimension uebergeben werden => nur Warning, aber komplett falsche
 * Adressierung der Elemente!!!
 */
int sum(int b[2][3]) {
	int i, j, sum = 0;
	for (i = 0; i < 2; i++)
		for (j = 0; j < 3; j++)
			sum += b[i][j];
	return sum;
}

/*
 * etwas verbesserte Variante gegenueber dem vorigen Bsp.; nun
 * kann die erste Dimension beliebig sein und angegeben werden;
 * die 2. Dimension muss jedoch nach wie vor 3 sein
 * => nicht geeignet fuer bel. Matrizen!!
 */
int sum1(int b[][3], int size) {
	int i, j, sum = 0;
	for (i = 0; i < size; i++)
		for (j = 0; j < 3; j++)
			sum += b[i][j];
	return sum;
}

/*
 * Allgemeine Loesung, falls die zweite Dimenion (und evtl. folgende
 * Dimensionen bei mehrdim. Arrays) nicht bekannt ist;
 * in diesem Fall muss man alle Dimensionen (s1, s2) uebergeben
 */
int sum2(int **b, int s1, int s2) {
	int i, j, sum = 0;
	for (i = 0; i < s1; i++)
		for (j = 0; j < s2; j++)
			/*
			 * Die Adressierung der Elemente muss nun komplett
			 * zu Fuss bzw. von Hand erfolgen;
			 * zunaechst "casten" wir b in einen int* (dies
			 * ist wichtig fuer die Zeigerarithmetik, da ja immer
			 * der Inkrement mit sizeof(TYP) multipliziert wird!);
			 * danach inkrementieren wir um i mal die Groesse eines
			 * Element-Arrays, also s2 (der Faktor 4 kommt impl. auch
			 * noch dazu, da der Zeiger ja Typ int hat), und dann noch
			 * um j;
			 * man vergleiche hierzu einfach die Adressen der Elemente,
			 * die ja von multi.c ausgegeben werden!
			 */
			sum += *((int*)b + i * s2 + j);
	return sum;
}

int main() {
	    /*
	     * analog zum vorigen Bsp.
	     */
	int a[2][3] = {{1,2,3},{4,5,6}};

	printf("summe = %d\n", sum(a));
	printf("summe = %d\n", sum1(a, sizeof(a) / sizeof(a[0])));
	    /*
	     * sizeof(a) / sizeof(a[0]) = (2*3*4) / (3*4) = 2
	     * ist die Groesse der ersten Dimension
	     *
	     * sizeof(a[0]) / sizeof(a[0][0]) = (3*4) / 4 = 3
	     * ist die Groesse der zweiten Dimension
	     *
	     * ;-)
	     */
	printf("summe = %d\n", sum2((int**)a, sizeof(a) / sizeof(a[0]), sizeof(a[0]) / sizeof(a[0][0])));

	return 0;
}
